module.exports = {
  db: 'mongo', // knex / mongodb
  name: 'person', // table name
  create: true, // able to add record,
  update: true, // able to update
  delete: true, // able to delete record(s)
  import: true, // can import from csv
  export: true, // can export to csv
  multiSelect: true, // multiple selection
  cols: {
    _id: { // primary key column, _id for mongodb (can cause error in mongodb)
      label: 'ID',
      auto: 'pk', // should override multiKey
      table: 'hide',
      filter: 'hide',
      add: 'hide',
      input: 'link',
      options: { from: 'table-person', to: 'table-grade-slot', relatedCol: 'personId' }, // router link
      edit: 'readonly'
    },
    firstName: {
      label: 'First Name',
      multiKey: true, // part of composite unique key - required!
      type: 'string', // string, integer, decimal, datetime, date, time, boolean
      input: 'textfield', // textfield, textarea, autocomplete, integer, decimal, select, multi-select, date, time, datetime, upload, link - to child table
      table: '', // truthy means hide column on table,
      filter: '', // truthy means hide column on filter,
      add: '', // 'hide', ''  - empty string means can show and edit
      edit: '', // 'hide', 'readonly', '' - empty string means can show and edit
      default: '', // a default value (today? now?)
      pattern: ''  // regex pattern
    },
    lastName: {
      label: 'Last Name',
      multiKey: true,
      type: 'string',
      input: 'textfield',
      // type ? text, password, etc...
      validation: {
        min: 0, max: 20
      }
    },
    sex: { // single select
      label: 'Sex',
      type: 'string',
      input: 'select',
      options: [
        { key: '', text: '' },
        { key: 'M', text: 'Male' },
        { key: 'F', text: 'Female' }
      ]
    },
    subjects: { // multi select
      label: 'Subjects',
      type: 'string',
      input: 'multi-select',
      options: [
        { key: 'EL1', text: 'English' },
        { key: 'EM', text: 'E Maths' },
        { key: 'AM', text: 'A Maths' },
        { key: 'PHY', text: 'Chemistry' },
        { key: 'CHEM', text: 'Physics' }
      ]
    },
    age: { // integer
      label: 'Age',
      type: 'integer',
      input: 'number',
      validation: {
        min: 10, max: 90
      }
    },
    gpa: { // decimal
      label: 'GPA',
      type: 'decimal',
      input: 'number',
      required: true
    },
    birthDate: { // date of birth - YYYY-MM-dd (no timezone - assume local - store as string)
      label: 'Brith date',
      type: 'string', // date
      input: 'date'
    },
    birthTime: { // HHmm (no timezone - assume local - store as string)
      label: 'Birth time',
      type: 'string', // time
      input: 'time'
    },
    country: {
      label: 'Country', // key text...
      type: 'string',
      input: 'autocomplete', // single select
      options: {
        parentCol: '', // use column to get parent value affecting a child
        parentTableColName: '', // the column name in the parent table
        childCol: 'state', // affect child column in this table
        tableName: 'country',
        limit:8,
        strict: true, // cannot enter own values, must be selected
        key: 'name',
        text: ''
      }
    },
    state: {
      label: 'State', // key text...
      type: 'string',
      input: 'autocomplete',
      options: {
        parentCol: 'country',
        parentTableColName: 'country_name',
        childCol: '',
        tableName: 'state',
        limit:8,
        key: 'name',
        text: ''
      }
    },
    birthDateTimeTz: { // date with time info
      label: 'Datetime with TZ',
      type: 'datetime',
      input: 'datetime'
    },
    website: { // test formatter?
      label: 'URL',
      type: 'string',
      input: 'textfield'
    },
    remarks: { // text area
      label: 'Remarks',
      type: 'string',
      input: 'textarea'
    },
    updated_by: {
      label: 'Updated By',
      auto: 'user', // autogenerated field... user = user id, ts = timestamp, pk = autogenerated primary key
      table: 'hide',
      filter: 'hide',
      add: 'hide',
      edit: 'readonly'
    },
    updated_at: {
      label: 'Updated At',
      auto: 'ts',
      table: 'hide',
      filter: 'hide',
      add: 'hide',
      edit: 'readonly'
    }
  },

  // generated values
  pk: '', // either pk column, or multikey if blank
  multiKey: [],
  required: [],
  auto: [],
  nonAuto: []
  // {id: 1, name: 'book1', categoryId: 1, rating: 5, yearPublished: '2004', created_at: mkDt() },
}